@@ -21,6 +21,9 @@ class TeaRequest:
     def __setattr__(self, key, value):
         if key in self._PROPERTY_DEFAULT_MAP:
             if not value:
-                self.__dict__[key] = self._PROPERTY_DEFAULT_MAP[key]
+                if isinstance(self._PROPERTY_DEFAULT_MAP[key], (list, dict)):
+                    self.__dict__[key] = self._PROPERTY_DEFAULT_MAP[key].copy()
+                else:
+                    self.__dict__[key] = self._PROPERTY_DEFAULT_MAP[key]
                 return
         self.__dict__[key] = value
